# SPDX-FileCopyrightText: 2023 Helge
# SPDX-FileCopyrightText: 2024 helge
#
# SPDX-License-Identifier: MIT

from .activity_factory import ActivityFactory
from .object_factory import Object


def test_activity_factory_like() -> None:
    activity_factory = ActivityFactory({"id": "actor_id", "followers": "followers"})

    like = activity_factory.like("target")
    like.to.add("target")
    like.cc.add("other")
    like.content = "🐮"

    result = like.build()

    assert result["@context"] == "https://www.w3.org/ns/activitystreams"

    assert result["to"] == ["target"]
    assert result["cc"] == ["other"]

    assert result["type"] == "Like"

    assert result.get("published")


def test_activity_factory_accept() -> None:
    activity_factory = ActivityFactory({"id": "actor_id", "followers": "followers"})

    accept = activity_factory.accept({"id": "obj"})
    result = accept.build()

    assert result["@context"] == "https://www.w3.org/ns/activitystreams"
    assert result["type"] == "Accept"
    assert result["actor"] == "actor_id"
    assert result["object"]["id"] == "obj"


def test_activity_factory_create():
    activity_factory = ActivityFactory({"id": "actor_id", "followers": "followers"})
    note = (
        Object(type="Note", attributed_to="actor_id", followers="account/followers")
        .as_public()
        .build()
    )

    result = activity_factory.create(note).build()

    assert result["cc"] == ["account/followers"]
    assert result["to"] == ["https://www.w3.org/ns/activitystreams#Public"]
    assert result["actor"] == "actor_id"


def test_activity_factory_create_two_attributed_to():
    activity_factory = ActivityFactory({"id": "actor_id", "followers": "followers"})
    note = (
        Object(
            type="Note",
            attributed_to=["actor_id", "other_actor"],
            followers="account/followers",
        )
        .as_public()
        .build()
    )

    result = activity_factory.create(note).build()

    assert result["cc"] == ["account/followers"]
    assert result["to"] == ["https://www.w3.org/ns/activitystreams#Public"]
    assert result["actor"] == "actor_id"


def test_activity_factory_create_no_cc() -> None:
    activity_factory = ActivityFactory({"id": "actor_id", "followers": "followers"})
    remote_actor = "https://abel/alice"
    note = Object(
        type="Note",
        attributed_to="actor_id",
        followers="account/followers",
        to={remote_actor},
    ).build()

    result = activity_factory.create(note).build()

    assert "cc" not in result
    assert result["to"] == [remote_actor]
    assert result["actor"] == "actor_id"


def test_activity_no_empty_to_cc() -> None:
    activity_factory = ActivityFactory({"id": "actor_id", "followers": "followers"})

    result = activity_factory.delete("target").build()

    assert set(result.keys()) == {"@context", "type", "object", "actor", "published"}



# SPDX-FileCopyrightText: 2023 Helge
# SPDX-FileCopyrightText: 2024 helge
#
# SPDX-License-Identifier: MIT

from dataclasses import dataclass


@dataclass
class Link:
    """Represents an ActivityStreams link object"""

    id: str | None = None
    type: str = "Link"
    media_type: str | None = None
    href: str | None = None
    name: str | None = None
    rel: str | None = None

    @staticmethod
    def for_object(obj):
        """Allows to create Object links following
        `FEP-e232 <https://codeberg.org/fediverse/fep/src/branch/main/fep/e232/fep-e232.md>`_
        """
        return Link(
            media_type='application/ld+json; profile="https://www.w3.org/ns/activitystreams"',
            href=obj,
            name=f"RE: {obj}",
        )

    def build(self):
        """Transform class to dict"""
        result = {
            "@context": "https://www.w3.org/ns/activitystreams",
            "type": self.type,
        }
        if self.media_type:
            result["mediaType"] = self.media_type
        if self.href:
            result["href"] = self.href
        if self.name:
            result["name"] = self.name
        if self.rel:
            result["rel"] = self.rel

        return result



# SPDX-FileCopyrightText: 2023 Helge
# SPDX-FileCopyrightText: 2024 helge
#
# SPDX-License-Identifier: MIT

import pytest

from .bovine_object import BovineObject


def test_invalid_context():
    with pytest.raises(ValueError):
        BovineObject({"@context": "https://www.w3.org/ns/activitystreams"})


@pytest.mark.parametrize(
    ("obj", "domain"),
    [({}, None), ({"type": "Note"}, None), ({"id": "https://remote"}, "local")],
)
def test_invalid_id_domain(obj, domain):
    with pytest.raises(ValueError):
        BovineObject(obj, domain=domain)


def test_correctly_parses_domain():
    bovine_object = BovineObject({"id": "https://remote"})

    assert bovine_object.domain == "remote"


@pytest.mark.parametrize(
    ("obj", "actor_id"),
    [
        ({}, None),
        ({"type": "Note"}, None),
        ({"actor": None}, None),
        ({"actor": {"type": "Actor"}}, None),
        ({"actor": "http://local/actor"}, "http://local/actor"),
        ({"actor": {"id": "http://local/actor"}}, "http://local/actor"),
    ],
)
def test_actor_id(obj, actor_id):
    bovine_object = BovineObject(obj, domain="local")

    if actor_id is None:
        assert bovine_object.actor_id is None
    else:
        assert bovine_object.actor_id == actor_id


@pytest.mark.parametrize(
    "obj",
    [
        {"actor": "http://remote/actor"},
        {"actor": {"id": "http://remote/actor"}},
    ],
)
def test_actor_id_error_on_different_domain(obj):
    bovine_object = BovineObject(obj, domain="local")

    with pytest.raises(ValueError):
        _ = bovine_object.actor_id


@pytest.mark.parametrize(
    ("obj", "object_id"),
    [
        ({}, None),
        ({"object": {}}, None),
        ({"something": "else"}, None),
        ({"object": "https://remote/object"}, "https://remote/object"),
        ({"object": "https://local/object"}, "https://local/object"),
        ({"object": {"id": "https://remote/object"}}, "https://remote/object"),
        ({"object": {"id": "https://local/object"}}, "https://local/object"),
    ],
)
def test_object_id(obj, object_id):
    bovine_object = BovineObject(obj, domain="local")

    if object_id is None:
        assert bovine_object.object_id is None
    else:
        assert bovine_object.object_id == object_id



# SPDX-FileCopyrightText: 2023 Helge
# SPDX-FileCopyrightText: 2024 helge
#
# SPDX-License-Identifier: MIT

from dataclasses import dataclass, field
from typing import List, Optional, Set

from bovine.utils import now_isoformat

from .utils import fediverse_handle_from_actor, id_for_object


@dataclass
class Object:
    """A dataclass representing an `ActivityStreams Object
    <https://www.w3.org/TR/activitystreams-vocabulary/#object-types>`_"""

    type: str
    attributed_to: Optional[str] = None
    followers: Optional[str] = None
    id: Optional[str] = None
    published: Optional[str] = None
    to: Set[str] = field(default_factory=set)
    cc: Set[str] = field(default_factory=set)

    name: Optional[str] = None
    summary: Optional[str] = None
    content: Optional[str] = None
    source: Optional[dict] = None

    width: Optional[int] = None
    height: Optional[int] = None

    in_reply_to: Optional[str] = None
    url: Optional[str] = None
    tag: List[dict] = field(default_factory=list)
    attachment: List[dict] = field(default_factory=list)
    href: Optional[str] = None
    icon: Optional[dict] = None
    media_type: Optional[str] = None

    def as_public(self):
        """makes the object public, i.e. public in to and followers in cc"""
        self.to.add("https://www.w3.org/ns/activitystreams#Public")
        if self.followers:
            self.cc.add(self.followers)
        return self

    def as_followers(self):
        """addresses the object to followers, if they are set"""
        if self.followers:
            self.to.add(self.followers)
        return self

    def as_unlisted(self):
        """makes the object unlisted, i.e. public in cc and followers in to"""
        if self.followers:
            self.to.add(self.followers)
        self.cc.add("https://www.w3.org/ns/activitystreams#Public")
        return self

    def now(self):
        """Sets published to now in isoformat"""
        self.published = now_isoformat()
        return self

    def build(self):
        """Returns the resulting object as a dictionary"""
        result = {
            "@context": "https://www.w3.org/ns/activitystreams",
            "type": self.type,
        }

        extra_fields = {
            "attributedTo": self.attributed_to,
            "to": list(self.to),
            "cc": list(self.cc - self.to),
            "id": self.id,
            "inReplyTo": self.in_reply_to,
            "published": self.published,
            "source": self.source,
            "name": self.name,
            "url": self.url,
            "summary": self.summary,
            "content": self.content,
            "tag": self.tag,
            "attachment": self.attachment,
            "href": self.href,
            "width": self.width,
            "height": self.height,
            "icon": self.icon,
            "mediaType": self.media_type,
        }

        for key, value in extra_fields.items():
            if value:
                result[key] = value

        if "to" in result and len(result["to"]) == 0:
            del result["to"]
        if "cc" in result and len(result["cc"]) == 0:
            del result["cc"]

        return result


class ObjectFactory:
    """ObjectFactory usually created through a BovineClient"""

    def __init__(self, actor_information=None, client=None):
        if client:
            self.client = client
            self.information = client.information
        elif actor_information:
            self.client = None
            self.information = actor_information
        else:
            raise TypeError(
                "You need to either specify actor_information or a BovineClient"
            )

    def note(self, **kwargs):
        """Creates a Note Object"""
        return Object(
            attributed_to=self.information["id"],
            type="Note",
            followers=self.information.get("followers"),
            published=now_isoformat(),
            **kwargs,
        )

    def reply(self, obj: dict, **kwargs):
        """Creates a reply for an object

        :param obj: Object being replied to"""
        cc = (set(obj.get("to", [])) | set(obj.get("cc", []))) - {
            self.information["id"]
        }
        return Object(
            attributed_to=self.information["id"],
            type=obj.get("type", "Note"),
            in_reply_to=obj.get("id"),
            followers=self.information.get("followers"),
            to={id_for_object(obj.get("attributedTo"))},
            cc=cc,
            published=now_isoformat(),
            **kwargs,
        )

    def article(self, **kwargs):
        """Creates an Article Object"""
        return Object(
            attributed_to=self.information["id"],
            type="Article",
            followers=self.information.get("followers"),
            published=now_isoformat(),
            **kwargs,
        )

    def event(self, **kwargs):
        """Creates an Event Object"""
        return Object(
            attributed_to=self.information["id"],
            type="Event",
            followers=self.information.get("followers"),
            published=now_isoformat(),
            **kwargs,
        )

    async def mention_for_actor_uri(self, actor_to_mention):
        """Creates a mention object for another actor. Requires client to be set.

        :param actor_to_mention: The URI of the actor to mention"""

        if not self.client:
            raise TypeError("client needs to be set at construction")

        remote_actor = await self.client.proxy(actor_to_mention)

        return Object(
            type="Mention",
            href=actor_to_mention,
            name=fediverse_handle_from_actor(remote_actor),
        )

    async def reply_with_mention(self, obj: dict, **kwargs):
        """Creates a reply for an object, mentioning the author of the original
        post. This is necessary for compatibility with Mastodon

        :param obj: Object being replied to"""
        cc = (set(obj.get("to", [])) | set(obj.get("cc", []))) - {
            self.information["id"]
        }
        original_author = id_for_object(obj.get("attributedTo"))
        mention = (await self.mention_for_actor_uri(original_author)).build()
        return Object(
            attributed_to=self.information["id"],
            type=obj.get("type", "Note"),
            in_reply_to=obj.get("id"),
            followers=self.information.get("followers"),
            to={original_author},
            cc=cc,
            tag=[mention],
            published=now_isoformat(),
            **kwargs,
        )



# SPDX-FileCopyrightText: 2023 Helge
# SPDX-FileCopyrightText: 2024 helge
#
# SPDX-License-Identifier: MIT

from dataclasses import dataclass
import logging

from typing import List

logger = logging.getLogger(__name__)


@dataclass
class Signature:
    """Helper class to parse HTTP Signatures"""

    key_id: str
    algorithm: str
    headers: str
    signature: str

    def __post_init__(self):
        if self.algorithm not in ["rsa-sha256", "hs2019"]:
            logger.error(f"Unsupported algorithm {self.algorithm}")
            logger.error(self.signature)
            logger.error(self.headers)
            logger.error(self.key_id)

            raise ValueError("Unsupported algorithm", self.algorithm)

    @property
    def fields(self) -> List[str]:
        """Returns the fields that are used when building the signature"""
        return self.headers.split(" ")

    @staticmethod
    def from_signature_header(header):
        """Takes the signature header and turns into Signature object

        The header is assumed of the for key=value,... The keys keyId,
        algorithm, headers, and signature are parsed. If algorithm
        is absent it is assumed to be rsa-sha256. The other keys are required.
        """
        headers = header.split(",")
        headers = [x.split('="', 1) for x in headers]
        parsed = {x[0]: x[1].replace('"', "") for x in headers}

        return Signature(
            parsed["keyId"],
            parsed.get("algorithm", "rsa-sha256"),
            parsed["headers"],
            parsed["signature"],
        )


def parse_signature_header(header):
    return Signature.from_signature_header(header)



# SPDX-FileCopyrightText: 2024 helge
#
# SPDX-License-Identifier: MIT

from dataclasses import dataclass, field
from typing import Set
from bovine.crypto.types import CryptographicIdentifier


@dataclass
class ReferencableCryptographicIdentifier(CryptographicIdentifier):
    """Allows one to track further meta data associated with a
    CryptographicIdentifier.

    :param id: The id of the object containing the identifier
    :param verification_relationships: Information for which use cases a cryptographic identifier is meant
    """

    id: str | None = None
    verification_relationships: Set[str] = field(default_factory=set)

    @classmethod
    def from_public_key(clz, data: dict):
        """Creates a ReferencableCryptographicIdentifier from a publicKey object, example:

        ```json
        {
            "id": "https://com.example/issuer/123#main-key",
            "owner": "https://com.example/issuer/123",
            "publicKeyPem": "-----BEGIN PUBLIC KEY-----\\n...\\n-----END PUBLIC KEY-----"
        }
        ```
        """
        result = super().from_public_key(data)
        result.id = data.get("id")
        return result

    @classmethod
    def from_multikey(clz, multikey: dict):
        """Creates an identifier from a multikey"""
        result = super().from_multikey(multikey)
        result.id = multikey.get("id")
        return result

    def with_verification_relationships(self, **kwargs):
        """Adds the verification relationships"""
        for relation_ship_name, ids in kwargs.items():
            if self.id in ids:
                self.verification_relationships.add(relation_ship_name)
        return self



# SPDX-FileCopyrightText: 2023 Helge
# SPDX-FileCopyrightText: 2024 helge
#
# SPDX-License-Identifier: MIT

import pytest
from unittest.mock import AsyncMock

from .object_factory import ObjectFactory


def test_object_factory() -> None:
    object_factory = ObjectFactory({"id": "actor_id", "followers": "followers"})

    note = object_factory.note().as_public()
    note.content = "text"
    result = note.build()

    assert set(result.keys()) == {
        "@context",
        "attributedTo",
        "content",
        "type",
        "to",
        "cc",
        "published",
    }

    assert result["to"] == ["https://www.w3.org/ns/activitystreams#Public"]
    assert result["cc"] == ["followers"]


def test_object_factory_now() -> None:
    object_factory = ObjectFactory({"id": "actor_id", "followers": "followers"})

    note = object_factory.note(content="text").as_public().now()
    result = note.build()

    assert set(result.keys()) == {
        "@context",
        "attributedTo",
        "published",
        "content",
        "type",
        "to",
        "cc",
    }

    assert result["to"] == ["https://www.w3.org/ns/activitystreams#Public"]
    assert result["cc"] == ["followers"]


async def test_mention_for_actor_uri() -> None:
    mock_client = AsyncMock()
    remote_uri = "https://remote/alice"

    mock_client.proxy.return_value = {
        "@context": "https://www.w3.org/ns/activitystreams",
        "id": remote_uri,
        "type": "Person",
        "preferredUsername": "alyssa",
    }

    object_factory = ObjectFactory(client=mock_client)

    mention_object = await object_factory.mention_for_actor_uri(remote_uri)
    mention = mention_object.build()

    assert set(mention.keys()) == {"@context", "type", "href", "name"}
    assert mention["type"] == "Mention"
    assert mention["href"] == remote_uri
    assert mention["name"] == "alyssa@remote"


@pytest.mark.parametrize(
    "remote_actor", ["remote_id", {"id": "remote_id", "type": "Person"}]
)
async def test_object_factory_reply(remote_actor) -> None:
    object_factory = ObjectFactory({"id": "actor_id", "followers": "followers"})

    note = {
        "@context": "https://www.w3.org/ns/activitystreams",
        "type": "Note",
        "id": "note_id",
        "attributedTo": remote_actor,
        "to": ["actor_id", "second_id"],
        "cc": ["third_id"],
    }

    reply = object_factory.reply(note).build()

    assert reply["attributedTo"] == "actor_id"
    assert reply["inReplyTo"] == "note_id"
    assert set(reply["to"]) == {"remote_id"}
    assert set(reply["cc"]) == {"second_id", "third_id"}



# SPDX-FileCopyrightText: 2023 Helge
# SPDX-FileCopyrightText: 2024 helge
#
# SPDX-License-Identifier: MIT

from datetime import datetime, timezone, timedelta
from typing import Optional, Tuple

from bs4 import BeautifulSoup

from bovine.models import JrdLink, JrdData

GMT_STRING = "%a, %d %b %Y %H:%M:%S GMT"


def pydantic_to_json(obj) -> dict:
    """Transforms a pydantic object from [bovine.models][bovine.models]
    into a dictionary, that can be serialized as json"""
    return obj.model_dump(mode="json", exclude_none=True)


def webfinger_response_json(account: str, url: str) -> dict:
    """helper to generate a webfinger response"""
    return pydantic_to_json(
        JrdData(
            subject=account,
            links=[JrdLink(href=url, rel="self", type="application/activity+json")],
        )
    )


def parse_fediverse_handle(account: str) -> Tuple[str, Optional[str]]:
    """Splits fediverse handle in name and domain Supported forms are:

    * user@domain -> (user, domain)
    * @user@domain -> (user, domain)
    * acct:user@domain -> (user, domain)
    """
    if account[0] == "@":
        account = account[1:]
    account = account.removeprefix("acct:")

    if "@" in account:
        user, domain = account.split("@", 1)
        return user, domain
    return account, None


def now_isoformat() -> str:
    """Returns now in Isoformat, e.g. "2023-05-31T18:11:35Z", to be used as the value
    of published"""
    return (
        datetime.now(tz=timezone.utc).replace(microsecond=0, tzinfo=None).isoformat()
        + "Z"
    )


def activity_pub_object_id_from_html_body(body: str) -> str | None:
    """Determines the object identifier from the html body
    by parsing it and looking for link tags with rel="alternate"
    and type application/activity+json"""

    soup = BeautifulSoup(body, features="lxml")
    element = soup.find(
        "link", attrs={"rel": "alternate", "type": "application/activity+json"}
    )
    if not element:
        return None

    return element.attrs.get("href")


def get_gmt_now() -> str:
    """Returns the current time in UTC as a GMT formatted string as used
    in the HTTP Date header"""
    return datetime.now(tz=timezone.utc).strftime(GMT_STRING)


def parse_gmt(date_string: str) -> datetime:
    """Parses a GMT formatted string as used in HTTP Date header"""
    return datetime.strptime(date_string, GMT_STRING).replace(tzinfo=timezone.utc)


def check_max_offset_now(dt: datetime, minutes: int = 5) -> bool:
    """Checks that offset of a datetime to now to be less than minutes"""

    now = datetime.now(tz=timezone.utc)

    if dt > now + timedelta(minutes=minutes):
        return False

    if dt < now - timedelta(minutes=minutes):
        return False

    return True



# SPDX-FileCopyrightText: 2023 Helge
# SPDX-FileCopyrightText: 2024 helge
#
# SPDX-License-Identifier: MIT

import pytest
import json
from unittest.mock import AsyncMock, MagicMock
from contextlib import asynccontextmanager

from . import Actor


@pytest.mark.parametrize(
    "actor",
    [
        {},
        {"@context": "about:bovine", "id": "https://local/actor"},
        {
            "@context": "about:bovine",
            "id": "https://local/actor",
            "inbox": "https://local/actor/inbox",
        },
        {
            "@context": "about:bovine",
            "id": "https://local/actor",
            "outbox": "https://local/actor/outbox",
        },
        {
            "@context": "about:bovine",
            "id": "https://local/actor",
            "inbox": "https://local/actor/inbox",
            "outbox": ["https://local/actor/outbox", "https://local/actor/another"],
        },
    ],
)
def test_actor_raises_exception(actor):
    with pytest.raises(ValueError):
        Actor(actor)


def test_actor_good_example():
    data = {
        "@context": "about:bovine",
        "id": "https://local/actor",
        "inbox": "https://local/actor/inbox",
        "outbox": "https://local/actor/outbox",
    }
    actor = Actor(data)
    assert isinstance(actor, Actor)

    assert actor.id == "https://local/actor"


@pytest.mark.parametrize(
    "extra",
    [
        {},
        {"preferredUsername": None},
        {"preferredUsername": ""},
        {"preferredUsername": ["alice", "bob"]},
    ],
)
def test_actor_acct_uri_bad_cases(extra):
    data = {
        "@context": "about:bovine",
        "id": "https://local/actor",
        "inbox": "https://local/actor/inbox",
        "outbox": "https://local/actor/outbox",
        **extra,
    }
    actor = Actor(data)
    assert actor.unvalidated_acct_uri is None


@pytest.mark.parametrize(
    "extra",
    [
        {"preferredUsername": "alice"},
        {"preferredUsername": {"@value": "alice", "@language": "@und"}},
    ],
)
def test_actor_acct_uri_good_cases(extra):
    data = {
        "@context": "about:bovine",
        "id": "https://local/actor",
        "inbox": "https://local/actor/inbox",
        "outbox": "https://local/actor/outbox",
        **extra,
    }
    actor = Actor(data)
    assert actor.unvalidated_acct_uri == "acct:alice@local"


async def test_actor_acct_uri_validation_good():
    data = {
        "@context": "about:bovine",
        "id": "https://local/actor",
        "inbox": "https://local/actor/inbox",
        "outbox": "https://local/actor/outbox",
        "preferredUsername": "alice",
    }
    actor = Actor(data)

    response = AsyncMock(__aenter__=AsyncMock())
    response.status = 200
    response.text.return_value = json.dumps(
        {
            "links": [
                {
                    "href": "https://local/actor",
                    "rel": "self",
                    "type": "application/activity+json",
                }
            ],
            "subject": "acct:alice@local",
        }
    )

    @asynccontextmanager
    async def response_context():
        yield response

    session = MagicMock()
    session.get.return_value = response_context()

    await actor.validate_acct_uri(session=session)

    assert "acct:alice@local" in actor.identifiers


async def test_actor_acct_uri_validation_bad():
    data = {
        "@context": "about:bovine",
        "id": "https://local/actor",
        "inbox": "https://local/actor/inbox",
        "outbox": "https://local/actor/outbox",
        "preferredUsername": "alice",
    }
    actor = Actor(data)

    response = AsyncMock(__aenter__=AsyncMock())
    response.status = 200
    response.text.return_value = json.dumps(
        {
            "links": [
                {
                    "href": "https://local/other",
                    "rel": "self",
                    "type": "application/activity+json",
                }
            ],
            "subject": "acct:alice@local",
        }
    )

    @asynccontextmanager
    async def response_context():
        yield response

    session = MagicMock()
    session.get.return_value = response_context()

    await actor.validate_acct_uri(session=session)

    assert "acct:alice@local" not in actor.identifiers



# SPDX-FileCopyrightText: 2023-2024 Helge
#
# SPDX-License-Identifier: MIT

from dataclasses import dataclass, field
from typing import Optional, Tuple, Dict, Any

from bovine.types import Visibility

from .activity_factory import ActivityFactory
from .object_factory import ObjectFactory
from .controller import Controller


def factories_for_actor_object(
    actor_profile: dict,
) -> Tuple[ActivityFactory, ObjectFactory]:
    """Builds activity and object factories from actor object

    :param actor_profile: The actor profile as a dictionary
    :return: Activity- and object factory
    """
    return ActivityFactory(actor_profile), ObjectFactory(
        actor_information=actor_profile
    )


@dataclass
class WithPublicKey:
    """Represents an object having a legacy public key specified under the `publicKey` key"""

    public_key: Optional[str] = None
    public_key_name: Optional[str] = None

    def build_public_key(self, id) -> dict | None:
        if self.public_key:
            return {
                "@context": "https://w3id.org/security/v1",
                "publicKey": {
                    "id": f"{id}#{self.public_key_name}",
                    "owner": id,
                    "publicKeyPem": self.public_key,
                },
            }
        return None


@dataclass
class Actor(WithPublicKey, Controller):
    """Actor class represents the basic ActivityStreams actor."""

    id: str | None = None
    type: str = "Person"
    name: Optional[str] = None
    preferred_username: Optional[str] = None
    inbox: Optional[str] = None
    outbox: Optional[str] = None
    followers: Optional[str] = None
    following: Optional[str] = None
    event_source: Optional[str] = None
    proxy_url: Optional[str] = None

    summary: Optional[str] = None
    icon: Optional[dict] = None
    url: Optional[str] = None

    properties: Dict[str, Any] = field(default_factory=dict)

    def combine_with_public_key(self, result):
        public_key = self.build_public_key(self.id)

        if public_key:
            result["@context"] = [result["@context"], public_key["@context"]]
            return {**public_key, **result}

        return result

    def combine_with_controller(self, result):
        controller = super().build()

        if "@context" in controller:
            result["@context"] = [result["@context"]] + controller["@context"]
            return {**controller, **result}

        return result

    def build(self, visibility=Visibility.PUBLIC):
        """Creates the json-ld representation of the actor."""
        result = {
            **self.properties,
            "@context": "https://www.w3.org/ns/activitystreams",
            "id": self.id,
            "type": self.type,
            **self._build_endpoints(visibility=visibility),
        }
        result = self.combine_with_public_key(result)
        result = self.combine_with_controller(result)

        if self.preferred_username:
            result["preferredUsername"] = self.preferred_username

        if visibility == Visibility.WEB:
            return result

        if self.name:
            result["name"] = self.name
        elif self.preferred_username:
            result["name"] = self.preferred_username

        for key, value in {
            "summary": self.summary,
            "icon": self.icon,
            "url": self.url,
        }.items():
            if value is not None:
                result[key] = value

        return result

    def _build_endpoints(self, visibility):
        result = {}

        if visibility == Visibility.WEB:
            return result

        if self.inbox:
            result["inbox"] = self.inbox
        else:
            result["inbox"] = self.id

        if self.outbox:
            result["outbox"] = self.outbox
        else:
            result["outbox"] = self.id

        if visibility != Visibility.OWNER:
            return result

        endpoints = self._build_user_endpoints()
        if endpoints:
            result["endpoints"] = endpoints

        if self.followers:
            result["followers"] = self.followers
        if self.following:
            result["following"] = self.following

        return result

    def _build_user_endpoints(self):
        endpoints = {}
        if self.event_source:
            endpoints["eventSource"] = self.event_source
        if self.proxy_url:
            endpoints["proxyUrl"] = self.proxy_url
        return endpoints


@dataclass
class Collection:
    id: str
    items: list

    def build(self) -> dict:
        return {
            "@context": "https://www.w3.org/ns/activitystreams",
            "id": self.id,
            "items": self.items,
            "type": "Collection",
        }


@dataclass
class OrderedCollection:
    id: str
    items: list | None = None
    count: int = 0
    first: str | None = None
    last: str | None = None

    def build(self) -> dict:
        result = {
            "@context": "https://www.w3.org/ns/activitystreams",
            "id": self.id,
            "totalItems": self.count,
            "type": "OrderedCollection",
        }

        if self.items:
            result["orderedItems"] = self.items

        if self.first:
            result["first"] = self.first

        if self.last:
            result["last"] = self.last

        return result


@dataclass
class OrderedCollectionPage:
    id: str
    items: list
    part_of: str
    next: str | None = None
    prev: str | None = None

    def build(self) -> dict:
        result = {
            "@context": "https://www.w3.org/ns/activitystreams",
            "id": self.id,
            "partOf": self.part_of,
            "orderedItems": self.items,
            "type": "OrderedCollectionPage",
        }

        if self.next:
            result["next"] = self.next

        if self.prev:
            result["prev"] = self.prev

        return result



# SPDX-FileCopyrightText: 2023 Helge
# SPDX-FileCopyrightText: 2024 helge
#
# SPDX-License-Identifier: MIT

from unittest.mock import AsyncMock

from .event_source import EventSource


async def test_event_source() -> None:
    session = AsyncMock()
    content = AsyncMock()
    response = AsyncMock(content=content)
    session.get.return_value = response
    response.raise_for_status = lambda: 1

    event_source = EventSource(session, "url")

    content.__aiter__.return_value = [b"data: text\n", b"\n", b"\n", b"\n", b"\n"]
    event = await event_source.__anext__()
    assert event
    assert event.data == "text"

    content.__aiter__.return_value = [b"\n", b"\n", b"data: text\n", b"\n"]
    event = await event_source.__anext__()
    assert event
    assert event.data == "text"



# SPDX-FileCopyrightText: 2023 Helge
# SPDX-FileCopyrightText: 2024 helge
#
# SPDX-License-Identifier: MIT

import aiohttp
from unittest.mock import AsyncMock

from .bearer import BearerAuthClient


async def test_bearer_auth_get():
    session = AsyncMock(aiohttp.ClientSession)
    session.get = AsyncMock()
    client = BearerAuthClient(session=session, bearer_key="token")

    await client.get("https://somewhere.test/path")

    session.get.assert_awaited_once()

    args = session.get.await_args

    assert args[0][0] == "https://somewhere.test/path"
    assert args[1]["headers"]["accept"] == "application/json"


async def test_bearer_auth_get_request_text_html():
    session = AsyncMock(aiohttp.ClientSession)
    session.get = AsyncMock()
    client = BearerAuthClient(session=session, bearer_key="token")

    await client.get("https://somewhere.test/path", headers={"accept": "text/html"})

    session.get.assert_awaited_once()

    args = session.get.await_args

    assert args[0][0] == "https://somewhere.test/path"
    assert args[1]["headers"]["accept"] == "text/html"



# SPDX-FileCopyrightText: 2023 Helge
# SPDX-FileCopyrightText: 2024 helge
#
# SPDX-License-Identifier: MIT

from .utils import host_target_from_url


def test_host_target_from_url():
    url = "https://test_domain/test_path"

    host, target = host_target_from_url(url)

    assert host == "test_domain"
    assert target == "/test_path"


def test_host_target_from_url_query():
    url = "https://test_domain/test_path?foo=bar#fragment"

    host, target = host_target_from_url(url)

    assert host == "test_domain"
    assert target == "/test_path?foo=bar"



# SPDX-FileCopyrightText: 2023 Helge
# SPDX-FileCopyrightText: 2024 helge
#
# SPDX-License-Identifier: MIT

from bovine.types import Visibility

from . import Actor


def test_actor() -> None:
    actor = Actor(id="http://example.com/actor/123")

    result = actor.build()

    assert set(result.keys()) == {"type", "id", "@context", "outbox", "inbox"}

    actor.name = "John Doe"
    result = actor.build()
    assert set(result.keys()) == {"type", "id", "name", "@context", "outbox", "inbox"}

    actor.public_key = "01234567890123456"
    actor.public_key_name = "key"
    result = actor.build()

    assert set(result.keys()) == {
        "type",
        "id",
        "name",
        "@context",
        "publicKey",
        "outbox",
        "inbox",
    }

    assert result["publicKey"] == {
        "id": actor.id + "#key",
        "owner": actor.id,
        "publicKeyPem": actor.public_key,
    }


def test_actor_second() -> None:
    actor = Actor(id="http://example.com/actor/123", inbox="inbox", outbox="outbox")

    result = actor.build()

    assert result["inbox"] == "inbox"
    assert result["outbox"] == "outbox"

    actor.proxy_url = "http://example.com/proxy"
    result = actor.build(visibility=Visibility.OWNER)

    assert result["endpoints"]["proxyUrl"] == "http://example.com/proxy"


def test_actor_with_icon() -> None:
    actor = Actor(id="http://example.com/actor/123", inbox="inbox", outbox="outbox")
    actor.icon = {"type": "Image"}

    result = actor.build()

    assert result["icon"] == {"type": "Image"}


def test_actor_with_properties() -> None:
    summary_map = {"de": "Schauspieler", "en": "actor"}
    actor = Actor(
        id="http://example.com/actor/123", properties={"summaryMap": summary_map}
    )

    result = actor.build()

    assert result["summaryMap"] == summary_map
